data {
  int<lower=0> N;                           // total number of observed functions
  int<lower=0> D;			// number of observations per function
  int I;									// number of subjects
  
  int Kp;                                  // number of PC basis functions
  matrix[Kp,D] Phis; 
  row_vector[D] YbeforeFPCs[N];	// N observed functions, each D long
  
  int group[I];			// which group is subject in  
  int which_re[N];
  real g_param;
  real dfs[Kp];			// Wishart dfs
  real mats[Kp];								// Wishart scale
}

transformed data {
 row_vector[Kp] Kp_zero;
 
 matrix[D,D] I_D;
 vector[D] D_one;
 
 
 for (d in 1:D){
   D_one[d] = 1;
 }
 I_D = diag_matrix(D_one); 
 
 for (k in 1:Kp){
   Kp_zero[k] = 0;
  }
}

parameters {
  row_vector[Kp] C[N];	                    // N vectors of scores, each Kp long
  real<lower=0.1> tau;						// measurement error variance
  //vector<lower=0.1>[Kp] g_tau;									
  matrix<lower=0.1>[1,1] g_tau[Kp];
  
  row_vector[Kp] gamma[2];						// variance of each group
  row_vector[Kp] g[I];							// random effects
}

transformed parameters {
  vector[Kp] g_tau_vector;
  
  for (k in 1:Kp){
    g_tau_vector[k] = g_tau[k,1,1];
  }
}

model { 
  row_vector[Kp] C_tau[N];

  tau ~ gamma(g_param, g_param); 
  
  for (k in 1:Kp){
    g_tau[k] ~ wishart(dfs[k], diag_matrix(rep_vector(mats[k], 1)));
  }
  
  for (i in 1:2){
    gamma[i] ~ normal(0, 10);     				// parametrization is with standard deviation
  }
  
  for (i in 1:I){
    g[i] ~ multi_normal_prec(Kp_zero, diag_matrix(g_tau_vector));
  }
  
  for (n in 1:N){
    C_tau[n] = exp(-gamma[group[which_re[n]]] - g[which_re[n]]);
  }
  
  for (n in 1:N) {
    C[n] ~ multi_normal_prec(Kp_zero, diag_matrix(C_tau[n]'));
  }
    
  for (n in 1:N) {
    YbeforeFPCs[n] ~ multi_normal_prec(C[n] * Phis, tau * I_D);
  }
}
