SimulateArtificialData <- function(score.seed, 
                                 num.subjects, 
                                 prop.group.1,
                                 num.motions.per.subject,
                                 error.sd,
                                 D,
                                 sim.alpha,
                                 sim.Kp,
                                 group.var.type, 
                                 pc.re.var=NULL, 
                                 re.spline.coef.var=NULL, 
                                 group.pc.var=NULL){
  set.seed(score.seed)
  # true score variance parameters
  group.pc.var.df <- data.frame(group=rep(0:1, each=4), 
                        PC=rep(1:4, 2), 
                        var=c(36, 12, 6, 4, 
                              18, 24, 12, 6))
  if (!is.null(group.pc.var)){
    group.pc.var.df$var <- group.pc.var
  }
  pc1 <- sin(seq(0, 2*pi, length.out=D))
  pc2 <- cos(seq(0, 2*pi, length.out=D))
  pc3 <- sin(seq(0, 4*pi, length.out=D))
  pc4 <- cos(seq(0, 4*pi, length.out=D))
  PCs <- cbind(pc1, pc2, pc3, pc4)
  library(pracma)
  PCs <- gramSchmidt(PCs)$Q
  PCs <- PCs[, 1:sim.Kp]
  num.motions <- num.subjects * num.motions.per.subject
  subject.indicator <- rep(1:num.subjects, each=num.motions.per.subject)
  group.indicator <- rep(1, num.motions)
  group.indicator[1:(num.subjects * prop.group.1 * num.motions.per.subject)] <- 0
  # mean random effect parameters, each row is a subject, each column is a PC 
  re.matrix <- sapply(unique(subject.indicator), 
                      function(x){1 * (subject.indicator==x)})
  if (!is.null(re.spline.coef.var)){
    subject.mean.res <- matrix(nrow=num.subjects, ncol=D)
    library(splines)
    df <- 10
    Theta = bs(1:D, df = df, intercept = TRUE, degree = 3)
    diff0 <- diag(10)
    diff2 <- diff(diag(D), diff=2)
    AtBtBA <- function(A, B){
      t(A) %*% t(B) %*% B %*% A
    }
    P <- sim.alpha * diff0 + (1 - sim.alpha) * AtBtBA(Theta, diff2)
    library(mvtnorm)
    set.seed(score.seed)
    recoefs <- rmvnorm(num.subjects, mean=rep(0, df), sigma=re.spline.coef.var * solve(P))
    subject.mean.res <- recoefs %*% t(Theta)
    mean.res <- re.matrix %*% subject.mean.res
  } else {
    mean.res <- matrix(0, nrow=num.motions, ncol=D)
    subject.mean.res <- NULL
  }
  var.scores <- matrix(nrow=num.motions, ncol=sim.Kp)
  if (!is.null(pc.re.var)){
    score.res <- matrix(nrow=num.subjects, ncol=sim.Kp)
    for (PC in 1:sim.Kp){
      score.res[, PC] <- rnorm(num.subjects, 0, sd=sqrt(pc.re.var[PC]))
    }
  } else {
    score.res <- matrix(0, nrow=num.subjects, ncol=sim.Kp)
  }
  for (PC in 1:sim.Kp){
    for (group in 0:1){
      var.scores[group.indicator==group, PC] <- 
        group.pc.var.df$var[group.pc.var.df$group==group & group.pc.var.df$PC==PC]
    }
    for (subject in 1:num.subjects){
      var.scores[subject.indicator==subject, PC] <- 
        var.scores[subject.indicator==subject, PC] * exp(score.res[subject, PC])
    }
  }
  C <- matrix(rnorm(prod(dim(var.scores)), 
              mean=0, 
              sd=sqrt(var.scores)), 
              ncol=sim.Kp)
  curves <- PCs %*% t(C)
  Y <- matrix(rnorm(prod(dim(curves)), mean=curves, sd=error.sd), 
              nrow=num.motions, byrow=T) + mean.res
  X <- matrix(1, nrow=nrow(Y))
  if (!is.null(re.spline.coef.var)){
    V <- re.matrix
  } else {
    V <- NULL
  }
  if (!is.null(pc.re.var)){
    het.model.matrix <- cbind(1-group.indicator, group.indicator, re.matrix)
    colnames(het.model.matrix) <- c("FE1_1", "FE2_1", paste0("RE1_", 1:num.subjects))
  } else {
    het.model.matrix <- cbind(1-group.indicator, group.indicator)
    colnames(het.model.matrix) <- c("FE1_1", "FE2_1")
  }
  if (!is.null(V)){
    random.effects <- as.data.frame(t(subject.mean.res)) %>%
    mutate(Time=1:D) %>% gather(Subject, Simulate, -Time)
  } else {
    random.effects <- NULL
  }
  FPCs <- GetFPCs(PCs)
  colnames(FPCs)[colnames(FPCs)=="Estimate"] <- "Simulate"
  var.fes <- group.pc.var.df %>% mutate(FPC=paste0("FPC", PC), Group=group+1, Simulate=log(var)) %>% select(-PC, -group, -var)
  if (!is.null(pc.re.var)){
    var.res <- as.data.frame(score.res) %>% mutate(Subject=paste0("V", 1:n())) %>% gather(FPC, Simulate, -Subject) %>% mutate(FPC=gsub("V", "FPC", FPC))
  } else {
    var.res <- NULL
  }
  return(list(curves=t(curves), 
              observed=Y,
              sim.random.effects=random.effects,
              sim.fpcs=FPCs,
              sim.var.fes=var.fes,
              sim.var.res=var.res,
              sim.scores=C,
              X=X, V=V, het.model.matrix=het.model.matrix, 
              PCs=PCs, 
              group.pc.var.df=group.pc.var.df,
              num.subjects=num.subjects,
              prop.group.1=prop.group.1,
              num.motions.per.subject=num.motions.per.subject,
              sim.alpha=sim.alpha,
              re.spline.coef.var=re.spline.coef.var,
              score.seed=score.seed, num.motions=num.motions, 
              subject.mean.res=subject.mean.res,
              var.scores=var.scores,
              score.res=score.res,
              pc.re.var=pc.re.var,
              error.sd=error.sd, D=D, C=C, 
              group.indicator=group.indicator,
              subject.indicator=subject.indicator,
              Kp=sim.Kp))
}
