library(rstan)

DoStan <- function(dat, n.iter, n.warmup, n.chains, dir){
  file.source <- "Model.stan"
  fit <- stan(model_name="FPCA", file = file.source, 
              data=dat, 
              iter = n.iter, warmup = n.warmup,
              control = list(adapt_delta = .65),
              chains = n.chains, seed=1, cores=n.chains)
  e <- rstan::extract(fit)
  ss <- summary(fit)$summary
  rhat <- ss[, 10]
  var.fes <- expand.grid(FPC=1:dat$Kp, Group=1:max(dat$group)) %>%
    mutate(term=paste0("gamma[", Group, ",", FPC, "]")) %>%
    mutate(Estimate=ss[term, "mean"], 
           SD=ss[term, "sd"], 
           Lower2.5=ss[term, "2.5%"], 
           Higher97.5=ss[term, "97.5%"], 
           FPC=paste0("FPC", FPC), 
           Converged=T, 
           Warning=NA, Code=NA)
  var.res <- as.data.frame(apply(e$g, c(2,3), mean)) %>% 
    mutate(Subject=paste0("V", 1:n())) %>%
    gather(FPC, Estimate, -Subject) %>%
    mutate(FPC=gsub("V", "FPC", FPC))
  return(list(rhat=rhat,
              var.fes=var.fes, var.res=var.res, time=time))
}
